function [Difference,LHS,RHS,R2like,JD_new,c_a_star,m_a_star,k_a_star,c_n_star,m_n_star,AC,AProb,P]  = Fsys(State,Stateminus,...
    Control_sparse,Controlminus_sparse,StateSS,...
    ControlSS,Gamma_state,Gamma_control,InvGamma,...
    par,mpar,grid,targets,Copula,P,aggrshock,oc)
% System of equations written in Schmitt-Grohé-Uribe generic form with states and controls
% STATE: Vector of state variables t+1 (only marginal distributions for histogram)
% STATEMINUS: Vector of state variables t (only marginal distributions for histogram)
% CONTROL: Vector of state variables t+1 (only coefficients of sparse polynomial)
% CONTROLMINUS: Vector of state variables t (only coefficients of sparse polynomial)
% STATESS and CONTROLSS: Value of the state and control variables in steady
% state. For the Value functions these are at full grids.
% GAMMA_STATE: Mapping such that perturbationof marginals are still
% distributions (sum to 1).
% GAMMA_CONTROL: Values of the polynomial base at all nodes to map
% sparse coefficient changes to full grid
% INVGAMMA: Projection of Value functions etc. to Coeffeicent space for
% sparse polynomials.
% PAR, MPAR: Model and numerical parameters (structure)
% GRID: Liquid, illiquid and productivity grid
% TARGETS: Stores targets for government policy
% COPULA: Interpolant that allows to map marginals back to full-grid
% distribuitions
% P: steady state transition matrix
% aggrshock: sets wether the Aggregate shock is TFP or uncertainty
%
% Authors: Christian Bayer and Ralph C. Luetticke, Bonn and London, March
% 2017%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================


%% Initializations
util  = @(c)(c.^(1-par.xi))./(1-par.xi);
mutil = @(c)(1./(c.^par.xi));
invutil = @(u)(((1-par.xi).*u).^(1/(1-par.xi)));
invmutil = @(mu)((1./mu).^(1/par.xi));
% Number of states, controls
nx   = mpar.nm+mpar.nk+mpar.nh -4 +2; % Number of states
ny   = length(ControlSS); % number of Controls
NxNx = nx-2; % Number of states without aggregate
NN   = mpar.nm*mpar.nh*mpar.nk; % Number of points in the full grid

% Initialize LHS and RHS
LHS  = zeros(nx+ny,1);
RHS  = zeros(nx+ny,1);

%% Indexes for LHS/RHS
% Indexes for Controls
Vind  = 1:NN;
mutil_cind = NN+(1:NN);
Vkind = 2*NN+(1:NN);
Qind  = 3*NN+1;
PIind = 3*NN+2;
Yind  = 3*NN+3;
Gind  = 3*NN+4;
Wind  = 3*NN+5;
Rind  = 3*NN+6;
Profitind  = 3*NN+7;
Nind  = 3*NN+8;
Tind  = 3*NN+9;
Kind  = 3*NN+10;
Bind  = 3*NN+11;

% Indexes for States
marginal_mind = (1:mpar.nm-1);
marginal_kind = (mpar.nm-1+(1:mpar.nk-1));
marginal_hind = (mpar.nm+mpar.nk-2 + (1:(mpar.nh-2)));

RBind = NxNx+1;
Sind  = NxNx+2;

%% Control Variables (Change Value functions according to sparse polynomial)
Control      = ControlSS .* (1+Gamma_control*(Control_sparse));
Controlminus = ControlSS .* (1+Gamma_control*(Controlminus_sparse));

Control(end-oc+1:end)       = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Control_sparse);
Controlminus(end-oc+1:end)  = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Controlminus_sparse);


%% State Variables
% read out marginal histogramm in t+1, t
Distribution      = StateSS(1:end-2) + Gamma_state * State(1:NxNx);
Distributionminus = StateSS(1:end-2) + Gamma_state * Stateminus(1:NxNx);

% Aggregate Endogenous States
RB      = StateSS(end-1) + (State(end-1));
RBminus = StateSS(end-1) + (Stateminus(end-1));

% Aggregate Exogenous States
S       = StateSS(end) + (State(end));
Sminus  = StateSS(end) + (Stateminus(end));

%% Split the Control vector into items with names
% Controls
V        = util(Control(Vind));
Vminus   = util(Controlminus(Vind));
Vk       = mutil(Control(Vkind));
Vkminus  = mutil(Controlminus(Vkind));
mutil_c       = mutil(Control(mutil_cind));
mutil_cminus  = mutil(Controlminus(mutil_cind));

% Aggregate Controls (t+1)
PI = exp(Control(PIind));
Y  = exp(Control(Yind ));
K  = exp(Control(Kind ));
L  = exp(Control(Bind ));

% Aggregate Controls (t)
PIminus = exp(Controlminus(PIind));
Qminus  = exp(Controlminus(Qind ));
Yminus  = exp(Controlminus(Yind ));
Gminus  = exp(Controlminus(Gind ));
Wminus  = exp(Controlminus(Wind ));
Rminus  = exp(Controlminus(Rind ));
Profitminus  = exp(Controlminus(Profitind ));
Nminus  = exp(Controlminus(Nind ));
Tminus  = exp(Controlminus(Tind ));
Kminus  = exp(Controlminus(Kind ));
Lminus  = exp(Controlminus(Bind ));

%% Write LHS values
% Controls
LHS(nx+Vind)       = invutil(Vminus);
LHS(nx+Vkind)      = invmutil(Vkminus);
LHS(nx+mutil_cind) = invmutil(mutil_cminus);
LHS(nx+Qind)       = (Qminus);
LHS(nx+Yind)       = (Yminus);

LHS(nx+Gind)       = (Gminus);
LHS(nx+Wind)       = (Wminus);
LHS(nx+Rind)       = (Rminus);
LHS(nx+Profitind)  = (Profitminus);
LHS(nx+Nind)       = (Nminus);
LHS(nx+Tind)       = (Tminus);
LHS(nx+Kind)       = (Kminus);
LHS(nx+Bind)       = (Lminus);

% States
% Marginal Distributions (Marginal histograms)
LHS(marginal_mind) = Distribution(1:mpar.nm-1);
ba=mpar.nm;
LHS(marginal_kind) = Distribution(ba+(1:mpar.nk-1));
ba=mpar.nm+mpar.nk;
LHS(marginal_hind) = Distribution(ba+(1:mpar.nh-2));
LHS(RBind)         = (RB);
LHS(Sind)          = (S);

% take into account that RB is in logs
RB=exp(RB); RBminus=exp(RBminus);

Bminus=Lminus - par.ABS*Kminus;
B=L - par.ABS*K;

%% Set of Differences for exogenous process
RHS(Sind) = (par.rhoS * (Sminus));

switch(aggrshock)
    case('TFP')
        TFP=exp(Sminus);
    case('Uncertainty')
        TFP=1;
        % Tauchen style for Probability distribution next period
        if any(Sminus)
            [P,~,~] = ExTransitions(exp(Sminus),grid,mpar,par);
        end
end

% Calculate aggregate Capital, Bonds and Human Capital Supply in t
marginal_m = Distribution(1:mpar.nm)';
marginal_k = Distribution((1:mpar.nk)+mpar.nm)';
marginal_h = Distribution(mpar.nm+mpar.nk+(1:mpar.nh));

marginal_mminus = Distributionminus(1:mpar.nm)';
marginal_kminus = Distributionminus((1:mpar.nk)+mpar.nm)';
marginal_hminus = Distributionminus(mpar.nm+mpar.nk+(1:mpar.nh))';

Hminus  = sum(grid.h(1:end-1).*marginal_hminus(1:end-1)); %Last column is entrepreneurs.
Lminus  = sum(grid.m.*marginal_mminus);

RHS(nx+Bind) = Lminus ;
RHS(nx+Kind) = (sum(grid.k.*marginal_kminus));

% Calculate joint distributions
cumdist = zeros(mpar.nm+1,mpar.nk+1,mpar.nh+1);
cumdist(2:end,2:end,2:end) = Copula({cumsum(marginal_mminus),cumsum(marginal_kminus)',cumsum(marginal_hminus)});
JDminus = diff(diff(diff(cumdist,1,1),1,2),1,3);

% Generate meshes for b,k,h
[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,grid.h);

%% Aggregate Output
mc              =  par.mu- (par.beta_PC * log(PI)*Y/Yminus - log(PIminus))/par.kappa;

RHS(nx+Nind)    =  (par.tau*TFP*par.alpha*Kminus.^(1-par.alpha).*mc).^(1/(1-par.alpha+par.gamma));

RHS(nx+Yind)    = (TFP*(Nminus).^(par.alpha).*Kminus.^(1-par.alpha));

%% Prices that are not part of Control Vector

% Wage Rate
RHS(nx+Wind) =  TFP *par.alpha       *mc.* (Kminus./(Nminus)).^(1-par.alpha);
% Return on Capital
RHS(nx+Rind) =  TFP *(1-par.alpha)   *mc.* ((Nminus)./Kminus).^(par.alpha)  - par.delta - ((RBminus + par.mortwedge)./PIminus-1).*par.ABS;

% Profits for Entrepreneurs
RHS(nx+Profitind) = (1-mc)*Yminus - Yminus.*(1/(1-par.mu))./par.kappa./2 .*log(PIminus).^2 + 1/2*par.phi*((K-Kminus).^2)./Kminus;

% Wages net of leisure services
WW=par.gamma/(1+par.gamma)*(Nminus./Hminus)*Wminus*ones(mpar.nm,mpar.nk,mpar.nh);
WW(:,:,end)=Profitminus*par.profitshare;

%% Incomes (grids)
inc.labor   = par.tau*WW.*(meshes.h);
inc.rent    = meshes.k*Rminus;
inc.capital = meshes.k*Qminus;
inc.money   = (RBminus/PIminus).*meshes.m...
    + (meshes.m<0).*(par.borrwedge/PIminus).*meshes.m...
    + (meshes.m>=0).*(meshes.m<=par.ABS.*meshes.k).*(par.mortwedge/PIminus).*meshes.m...
    + (meshes.m>=0).*(meshes.m>par.ABS.*meshes.k).*(par.mortwedge*par.ABS.*meshes.k/PIminus);

%% First Set: Value Functions, Marginal Values
%% Update policies

EVk = reshape(reshape(Vk,[mpar.nm*mpar.nk mpar.nh])*P',[mpar.nm,mpar.nk mpar.nh]);
RBaux = RB/PI + (meshes.m<0).*(par.borrwedge/PI)...
      + (par.ABS>0).*(meshes.m>=0).*(meshes.m<=par.ABS.*meshes.k).*(par.mortwedge/PI);
EVm = reshape(reshape(RBaux(:).*mutil_c,[mpar.nm*mpar.nk mpar.nh])*P',[mpar.nm,mpar.nk mpar.nh]);

[c_a_star,m_a_star,k_a_star,c_n_star,m_n_star] = EGM_policyupdate(EVm,EVk,Qminus,PIminus,RBminus,inc,meshes,grid,par,mpar);

%% Update Value Functions
meshaux=meshes;
meshaux.h(:,:,end)=1000;

EV    = reshape(V,[mpar.nm*mpar.nk, mpar.nh])*P';
Vnext = griddedInterpolant(meshaux.m,meshaux.k,meshaux.h,reshape(EV,[mpar.nm mpar.nk mpar.nh]));


VA    = util(c_a_star) + par.beta*Vnext(m_a_star,k_a_star,meshaux.h);
VN    = util(c_n_star) + par.beta*Vnext(m_n_star,meshaux.k,meshaux.h);

AProb = max(min(1./(1+exp(-((VA-VN)-par.mu_chi)./par.sigma_chi)),1-1e-6),1e-6);
AC =  par.sigma_chi.*((1-AProb(:)).*log(1-AProb(:)) + AProb(:).*log(AProb(:)))...
    + par.mu_chi.*AProb(:) -  (par.mu_chi-par.sigma_chi*log(1+exp(par.mu_chi/par.sigma_chi)));
AProb = AProb.*par.nu;
AC    = AC.*par.nu;

Vaux  = AProb(:).*VA(:) + (1-AProb(:)).*VN(:) - AC(:); % Calculate new Value Function

RHS(nx+Vind)= invutil(Vaux(:)); % Write new Value function to RHS of Difference F

%% Update Marginal Value Bonds
mutil_c_n = mutil(c_n_star); % marginal utility at consumption policy no adjustment
mutil_c_a = mutil(c_a_star); % marginal utility at consumption policy adjustment
mutil_c_aux    = AProb.*mutil_c_a + (1-AProb).*mutil_c_n; % Expected marginal utility at consumption policy (w &w/o adjustment)
RHS(nx+mutil_cind) = invmutil(mutil_c_aux(:)); % Write Marginal Utility to RHS of F

%% Update marginal Value of Capital
EVk     = reshape(Vk,[mpar.nm*mpar.nk, mpar.nh])*P';
Vk_next = griddedInterpolant(meshaux.m,meshaux.k,meshaux.h,reshape(EVk,[mpar.nm mpar.nk mpar.nh]));
Vk_aux  = AProb.*(Rminus+Qminus).*mutil_c_a + (1-AProb).*Rminus.*mutil_c_n+ par.beta.*(1-AProb).*Vk_next(m_n_star,meshaux.k,meshaux.h); % Expected marginal utility at consumption policy (w &w/o adjustment)
RHS(nx+Vkind) = invmutil(Vk_aux(:));  % Write Marginal Value of K to RHS of F


%% Differences for distributions

% Transitions: find next smallest on-grid value for money and capital choices
[Dist_m_a,idm_a] = genweight(m_a_star,grid.m);
[Dist_m_n,idm_n] = genweight(m_n_star,grid.m);
[Dist_k,idk_a]   = genweight(k_a_star,grid.k);

% Iterate Distribution function forward
JD_new = JDIteration_mex(reshape(JDminus,[mpar.nm, mpar.nk, mpar.nh]),...
    int32(idm_n),int32(idm_a),int32(idk_a),Dist_m_n,Dist_m_a,Dist_k,...
    reshape(AProb,[mpar.nm, mpar.nk, mpar.nh]),P);

JD_new = reshape(JD_new(:),[mpar.nm,mpar.nk,mpar.nh]);

% Next period marginal histograms
% liquid assets
aux_m = squeeze(sum(sum(JD_new,2),3));
RHS(marginal_mind) = aux_m(1:end-1); %Leave out last state
% illiquid assets
aux_k =  squeeze(sum(sum(JD_new,1),3))' ;
RHS(marginal_kind) = aux_k(1:end-1); %Leave out last state
% human capital
aux_h = squeeze(sum(sum(JD_new,1),2));
RHS(marginal_hind) = aux_h(1:end-2); %Leave out last state & entrepreneurs

%% Third Set: Government Budget constraint
% Return on bonds (Taylor Rule)
RHS(RBind) = log(par.RB) + par.rho_R* log(RBminus/par.RB)  + log(PIminus/par.PI).*((1-par.rho_R)*par.theta_pi);

taxrevenue =(1-par.tau).*Wminus.*Nminus +(1-par.tau).*Profitminus;

% Inflation jumps to equilibrate real bond supply and demand
RHS(nx+PIind) = par.rho_B * log((Bminus)/(targets.B)) ...
    + par.rho_B * log(RBminus/par.RB)...
    - (par.rho_B+par.gamma_pi) * log(PIminus/par.PI) ...
    - par.gamma_T * log((Tminus)/(targets.T));
LHS(nx+PIind) = log((B)/(targets.B));

% Government expenditures
RHS(nx+Gind) =  B - Bminus*RBminus/PIminus + Tminus;

RHS(nx+Tind) = taxrevenue;

% Resulting Price of Capital
RHS(nx+Qind)=(par.phi*(K./Kminus-1)+1)-par.ABS;

%% Difference
Difference=InvGamma(:,:)*((LHS-RHS)./[ones(nx,1);(ControlSS(1:end-oc));ones(oc,1)]);

%% Calculate check on quality of approximation of Distribution
cumdist = zeros(mpar.nm+1,mpar.nk+1,mpar.nh+1);
cumdist(2:end,2:end,2:end) = Copula({cumsum(marginal_m),cumsum(marginal_k)',cumsum(marginal_h)});
JD= diff(diff(diff(cumdist,1,1),1,2),1,3);
IMSE   = sum((JD_new(:)-JD(:)).^2);
R2like = 1-IMSE/sum((JD_new(:)-1./numel(JD_new)).^2);

end
function [ weight,index ] = genweight( x,xgrid )
% function: GENWEIGHT generates weights and indexes used for linear interpolation
%
% X: Points at which function is to be interpolated.
% xgrid: grid points at which function is measured
% no extrapolation allowed
[~,index] = histc(x,xgrid);
index(x<=xgrid(1))=1;
index(x>=xgrid(end))=length(xgrid)-1;

weight = (x-xgrid(index))./(xgrid(index+1)-xgrid(index)); % weight xm of higher gridpoint
weight(weight<=0) = 1.e-16; % no extrapolation
weight(weight>=1) = 1-1.e-16; % no extrapolation

end  % function
