function [denHaanTable]=denHaanSimulation(par, mpar, targets, grid, aggrshock)
% DenHaan Errors

%% Search for a new steady state with equi-distant income grid on +/- 1 long run stdev. to capture the ergodic distribution of income risk
% in the denHaan simulation. The income grid is kept relatively small to assure that after arbitrarily long realizations of low income risk 
%the Markov process for income is still irreducible.   
% (Importance sampling is used for the baseline steady-state and Reiter method because it best approximates the steady-state 
% idiosyncratic income process for a given number of gridpoints. The Reiter method only perturbs around this steady-state process.) 
mpar.ns_Tauchen=3;
mpar.nK=1;
mpar.nM=1;
mpar.ns=1;
mpar.nh=26;
mpar.tauchen='equi';

%% Solve for SS with equi-spaced idiosyncratic income grid

% Use Tauchen method to approximate idiosyncratic income process
[P_H,grid,par]= stochastics_variance(par, mpar, grid);
% Use Rouwenhorst method to approximate stochastic volatility 
[P_S, grid.s] = rouwen(par.rhoS, 0, par.sigmaS/sqrt(1-par.rhoS^2), mpar.ns_Tauchen);
grid.s=exp(grid.s');

% Solve for steady state capital by bi-section
[c_n_guess,m_n_star,c_a_guess,m_a_star,cap_a_star,psi_guess,joint_distr, AProb,V,R_fc,W_fc,Profits_fc,Output,grid]...
    =steadystate(P_H,grid,mpar,par);

% Calculate steady state capital and further statistics
[mesh.k,mesh.m]=meshgrid(grid.k,grid.m);
clear targets
targets.ShareBorrower=sum((grid.m<0).*sum(sum(joint_distr,2),3)');
targets.K=sum(grid.k.*sum(sum(joint_distr,1),3));
targets.B=grid.m*sum(sum(joint_distr,2),3)-par.ABS*targets.K;
Liquid  = max(min(mesh.m,0),mesh.m-mesh.k*par.ABS);
Illiquid =(mesh.k*par.Q+min(max(mesh.m,0),mesh.k*par.ABS));
Mortgage = max(0,mesh.k*par.ABS-max(mesh.m,0));
JDredux = sum(joint_distr,3);
targets.LI=sum(JDredux(:).*Liquid(:));
targets.IL=sum(JDredux(:).*Illiquid(:));
targets.BoverK     = targets.B/targets.K;
targets.LIoverIL   = targets.LI/targets.IL;
targets.mortgShare = sum(JDredux(:).*Mortgage(:))./targets.K;
targets.AdjProb=AProb(:)'*joint_distr(:);
targets.StdAdjProb=(sum((AProb(:)).^2.*joint_distr(:))-targets.AdjProb.^2)^0.5;

targets.L=grid.N*sum(grid.h'.*squeeze(sum(sum(joint_distr,1),2)));
targets.KY=targets.K/Output;
targets.BY=targets.B/Output;
targets.LY=targets.L/Output;
targets.Y=Output;
BCaux_M=sum(sum(joint_distr,2),3);
targets.m_bc=BCaux_M(1,:);
targets.m_0=BCaux_M(grid.m==0);
BCaux_K=sum(sum(joint_distr,1),3);
targets.k_bc=BCaux_K(:,1);
aux_MK=sum(joint_distr,3);
targets.WtH_b0=sum(aux_MK(Liquid==0 & Illiquid>0));
targets.WtH_bnonpos=sum(aux_MK(Liquid<=0 & Illiquid>0));

targets.T =(1-par.tau).*W_fc.*grid.N +(1-par.tau).*Profits_fc;
par.gamma1=targets.B*(1-par.RB)+targets.T;
par.R=R_fc - ((par.RB + par.mortwedge)./par.PI-1).*par.ABS;
par.W=W_fc(1);
par.PROFITS=Profits_fc(1);
par.N=grid.N;
targets.GtoY=par.gamma1/Output;

% Calculate Value Functions
grid.K=sum(grid.k.*sum(sum(joint_distr,1),3));
grid.B=sum(grid.m.*sum(sum(joint_distr,2),3)');

[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,grid.h);

% Calculate Marginal Values of Capital (k) and Liquid Assets(m)
RBRB = par.RB/par.PI + (meshes.m<0).*(par.borrwedge/par.PI)...
    + (par.ABS>0).*(meshes.m>=0).*(meshes.m<=par.ABS.*meshes.k).*(par.mortwedge/par.PI);

% Liquid Asset
mutil_c_n = 1./(c_n_guess.^par.xi); % marginal utility at consumption policy no adjustment
mutil_c_a = 1./(c_a_guess.^par.xi); % marginal utility at consumption policy adjustment
mutil_c = AProb.*mutil_c_a + (1-AProb).*mutil_c_n; % Expected marginal utility at consumption policy (w &w/o adjustment)
Vm = RBRB.*mutil_c; %take return on money into account
Vm = reshape(reshape(Vm,[mpar.nm*mpar.nk mpar.nh]),[mpar.nm,mpar.nk mpar.nh]);

% Capital
Vk = AProb.*(par.R+par.Q).*mutil_c_a + (1-AProb).*par.R.*mutil_c_n + (1-AProb).*psi_guess; % Expected marginal utility at consumption policy (w &w/o adjustment)
Vk = reshape(reshape(Vk,[mpar.nm*mpar.nk mpar.nh]),[mpar.nm,mpar.nk mpar.nh]);

% Produce non-parametric Copula
cum_dist = cumsum(cumsum(cumsum(joint_distr,1),2),3);
marginal_m = cumsum(squeeze(sum(sum(joint_distr,2),3)));
marginal_k = cumsum(squeeze(sum(sum(joint_distr,1),3)));
marginal_h = cumsum(squeeze(sum(sum(joint_distr,2),1)));

Copula = griddedInterpolant({marginal_m,marginal_k',marginal_h},cum_dist,'spline');

%% Solve model 
mpar.maxdim=12;

mainskript_statereduc

disp('Computing system for SS.');
toc
F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

%% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

%% Simulation Preparation
x0=zeros(numstates,1);
MX=[eye(length(x0));gx];
IRF_state_sparse=[];
Prices=[];
x=x0;

mpar.maxlag=1000;

PS_S=cumsum(P_S,2);
pr_s=rand(RandStream('mcg16807', 'Seed',20180525),1,mpar.maxlag);

JDminus=joint_distr;
RBminus=log(par.RB);
Krealized_dH=grid.K;
Brealized_dH=grid.B;
Hrealized_dH=par.H;
MU=[];
ss(1)=2;
XssRED=[squeeze(sum(sum(joint_distr(1:end,:,:),2),3)); ... % marginal distribution liquid
    squeeze(sum(sum(joint_distr(:,1:end,:),1),3))'; ... % marginal distribution illiquid
    squeeze(sum(sum(joint_distr(:,:,1:end),2),1)); ... % marginal distribution productivity
    log(par.RB); 0];

PIGamma= pinv(Gamma2);

%% Simulation
for t=1:mpar.maxlag
    ControlNOW=(gx*x)';
    xFOR=hx*x;
    ControlFOR=(gx*xFOR)';
    
    [JDminus,RBminus,Prices(t,:),MU]  = denHaansys_Prices(JDminus,RBminus,log(grid.s(ss(t))),...
        ControlFOR',  ControlNOW',...
        Yss,Gamma_control,par,mpar,grid,P_H,aggrshock,oc,targets);
    
    Krealized_dH(t+1)=grid.k*sum(sum(JDminus,1),3)';
    JDHaux=squeeze(sum(sum(JDminus,1),2));
    Hrealized_dH(t+1)=grid.h(1:end-1)*JDHaux(1:end-1);
    Brealized_dH(t+1)=grid.m*sum(sum(JDminus,2),3);
    % liquid assets
    aux_m = squeeze(sum(sum(JDminus,2),3));
    % illiquid assets
    aux_k =  squeeze(sum(sum(JDminus,1),3))' ;
    % human capital
    aux_h = squeeze(sum(sum(JDminus,1),2));
    Xaux=([aux_m(1:end);aux_k(1:end);aux_h(1:end)]-XssRED(1:(mpar.nm+mpar.nk+mpar.nh)));
    
    x(1:end-2)=PIGamma*Xaux;
    x(end-1)=RBminus-log(par.RB);
    ss(t+1) = min(mpar.ns_Tauchen,sum(PS_S(ss(t),:)<pr_s(t))+1);
    x(end)=log(grid.s(ss(t+1)));
    
end

%% Linear Simulation

mpar.maxlag=t;

xlinear=x0;
IRF_state_sparse=[];

for tt=1:mpar.maxlag
    xlinear(end)= log(grid.s(ss(tt)));
    IRF_state_sparse(:,tt)=(MX*xlinear)';
    xlinear=hx*xlinear;
end

%% Plotting
IRF_distr  = Gamma_state*IRF_state_sparse(1:numstates-2,1:mpar.maxlag);
clear Stateguess
% preparation
Stateguess(1,:)=       IRF_state_sparse(n2(2)+2,:);
Stateguess(2,:)=grid.m*IRF_distr((1:mpar.nm),:)    + grid.B;
Stateguess(3,:)=grid.k*IRF_distr(mpar.nm+(1:mpar.nk),:) + grid.K;
Stateguess(4,:)=grid.h(1:end-1)*IRF_distr(mpar.nm+mpar.nk+(1:mpar.nh-1),:) + par.H;
Stateguess(5,:)=       IRF_state_sparse(n2(2)+1,:) + log(par.RB);

PIlinear=1+IRF_state_sparse(end-oc+2,1:end);
Qlinear=par.Q*(1+IRF_state_sparse(end-oc+1,1:end));
PIrealized=Prices(:,2);
Qrealized=Prices(:,1);
logPI=log(PIrealized(:)./PIlinear(:));
logQ=log(Qrealized(:)./Qlinear(:));

Klinear=Stateguess(3,:);
Blinear=Stateguess(2,:);
Hlinear=Stateguess(4,:);

logK=log(Krealized_dH(1:end-1)./Klinear);
logB=log(Brealized_dH(1:end-1)./Blinear);
logH=log(Hrealized_dH(1:end-1)/1.0141./Hlinear);

upto=1000;
denHaanTable(1,1)=mean(100*[abs(logQ(1:upto))]);
denHaanTable(2,1)=max(100*[abs(logQ(1:upto))]);
denHaanTable(1,2)=mean(100*[abs(logK(1:upto))]);
denHaanTable(2,2)=max(100*[abs(logK(1:upto))]);
denHaanTable(1,3)=mean(100*[abs(logPI(1:upto))]);
denHaanTable(2,3)=max(100*[abs(logPI(1:upto))]);
denHaanTable(1,4)=mean(100*[abs(logB(1:upto))]);
denHaanTable(2,4)=max(100*[abs(logB(1:upto))]);
