function [JD,RB,EqQuant,c_a,m_a,k_a,c_n,m_n,AC,AProb,P,MU]  = denHaansys_Prices(JDminus,RBminus,Sminus,...
    Control_F,Control_GUESS,ControlSS,Gamma_control,par,mpar,grid,P,aggrshock,oc,targets)
% denHaansys iterates the states forward using expected prices and value funtions
% that stored in the forecast CONTROL_F. It uses prices as stored in Control_GUESS
% as starting guesses and finds the within period market clearing prices.
%
% GAMMA_CONTROL maps polynomial coefficients to the value function (and its marginals),
% see Fsys. Parameters are stored in  par and mpar.
% JDminus, RBminus and Sminus contain the time t state variables.
% The function returns time t+1 state variables JD (distribution), RB (CB interest rate)
% and S (uncertainty state), along policy functions (c_a,m_a,k_a,c_n,m_n).
%
%
% Authors: Christian Bayer and Ralph C. Luetticke, Bonn and London, March
% 2017%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================


%% Initializations
util  = @(c)(c.^(1-par.xi))./(1-par.xi);
mutil = @(c)(1./(c.^par.xi));

% Number of states, controls
NN   = mpar.nm*mpar.nh*mpar.nk; % Number of points in the full grid


%% Indexes for LHS/RHS
% Indexes for Controls
Vind  = 1:NN;
mutil_cind = NN+(1:NN);
Vkind = 2*NN+(1:NN);
Qind  = 3*NN+1;
PIind = 3*NN+2;
Yind  = 3*NN+3;

%% Control Variables (Change Value functions according to sparse polynomial)
Control      = ControlSS .* (1+Gamma_control*(Control_F));
Controlminus = ControlSS .* (1+Gamma_control*(Control_GUESS));

Control(end-oc+1:end)       = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Control_F);
Controlminus(end-oc+1:end)  = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Control_GUESS);


%% Split the Control vector into items with names
% Controls
V        = util(Control(Vind));
Vk       = (mutil(Control(Vkind)));
mutil_c  = (mutil(Control(mutil_cind)));

% Aggregate Controls (t+1)
PI = exp(Control(PIind));
Y  = exp(Control(Yind ));
% Aggregate Controls (t)
PIminus = exp(Controlminus(PIind));
Qminus  = exp(Controlminus(Qind ));
Yminus  = exp(Controlminus(Yind ));

% States
% Marginal Distributions (Marginal histograms)

% Calculate aggregate Capital, Bonds and Human Capital Supply in t
marginal_hminus=squeeze(sum(sum(JDminus,1),2));
marginal_kminus=squeeze(sum(sum(JDminus,1),3));
marginal_bminus=squeeze(sum(sum(JDminus,2),3));

Hminus  = grid.h(1:end-1)*marginal_hminus(1:end-1); %Last column is entrepreneurs.
Kminus  = sum(grid.k.*marginal_kminus);
Bminus  = sum(grid.m.*marginal_bminus');

% take into account that RB is in logs
RBminus=exp(RBminus);

switch(aggrshock)
    case('Uncertainty')
        [P,~,~] = ExTransitions(exp(Sminus),grid,mpar,par);
end

DD = @(Qg)PriceD(Qg,V,Vk,PI,Y/Yminus,Bminus,Kminus,Hminus,RBminus,Sminus,mutil_c,mpar,par,grid,aggrshock,P,JDminus,targets);
QuantGuess=[Qminus,PIminus];

options=optimset('Display','off','TolFun',1e-10,'TolX',1e-10);

[EqQuant]=fsolve(@(p)DD(p),QuantGuess,options);
[~,JD,RB,c_a,m_a,k_a,c_n,m_n,AC,AProb,MU]= DD(EqQuant);
fprintf('%2.2f %8.2f \n',(abs(EqQuant-QuantGuess)./QuantGuess)*100);

end

function [QuantDiff,JD,RB,c_a_star,m_a_star,k_a_star,c_n_star,m_n_star,AC,AProb,MU] = PriceD(QuantGuess,V,Vk,PI,YG,Bminus,Kminus,Hminus,RBminus,Sminus,mutil_c,mpar,par,grid,aggrshock,P,JDminus,targets)
Qminus=QuantGuess(1);
PIminus=QuantGuess(2);

K  = ((Qminus-1+par.ABS)/par.phi+1)*Kminus;

RB = log(par.RB) + par.rho_R* log(RBminus/par.RB)  + log(PIminus/par.PI).*((1-par.rho_R)*par.theta_pi);

util  = @(c)(c.^(1-par.xi))./(1-par.xi);


%% Aggregate Output
switch(aggrshock)
    case('TFP')
        TFP=exp(Sminus);
    case('Uncertainty')
        TFP=1;
end

mc        =  par.mu- (par.beta * log(PI)*YG - log(PIminus))/par.kappa;

Nminus    =  (par.tau*TFP*par.alpha*Kminus.^(1-par.alpha).*mc).^(1/(1-par.alpha+par.gamma));

YminusNEW = (TFP*(Nminus).^(par.alpha).*Kminus.^(1-par.alpha));

%% Prices that are not part of Control Vector

% Wage Rate
Wminus =  TFP *par.alpha       *mc.* (Kminus./(Nminus)).^(1-par.alpha);
% Return on Capital
Rminus =  TFP *(1-par.alpha)   *mc.* ((Nminus)./Kminus).^(par.alpha)  - par.delta - ((RBminus + par.mortwedge)./PIminus-1).*par.ABS;

% Profits for Entrepreneurs
Profitminus = (1-mc)*YminusNEW - YminusNEW.*(1/(1-par.mu))./par.kappa./2 .*log(PIminus).^2 + 1/2*par.phi*((K-Kminus).^2)./Kminus;

taxrev   = (1-par.tau).*Wminus.*Nminus +(1-par.tau).*Profitminus;

% Generate meshes for b,k,h
[meshes.m,meshes.k,meshes.h] = ndgrid(grid.m,grid.k,grid.h);

% Wages net of leisure services
WW=par.gamma/(1+par.gamma)*(Nminus./Hminus)*Wminus*ones(mpar.nm,mpar.nk,mpar.nh);
WW(:,:,end)=Profitminus*par.profitshare;

%% Incomes (grids)
inc.labor   = par.tau*WW.*(meshes.h);
inc.rent    = meshes.k*Rminus;
inc.capital = meshes.k*Qminus;
inc.money   = (RBminus/PIminus).*meshes.m...
    + (meshes.m<0).*(par.borrwedge/PIminus).*meshes.m...
    + (meshes.m>=0).*(meshes.m<=par.ABS.*meshes.k).*(par.mortwedge/PIminus).*meshes.m...
    + (meshes.m>=0).*(meshes.m>par.ABS.*meshes.k).*(par.mortwedge*par.ABS.*meshes.k/PIminus);

%% First Set: Value Functions, Marginal Values
%% Update policies

EVk = reshape(reshape(Vk,[mpar.nm*mpar.nk mpar.nh])*P',[mpar.nm,mpar.nk mpar.nh]);
RBaux = exp(RB)/PI + (meshes.m<0).*(par.borrwedge/PI)...
      + (par.ABS>0).*(meshes.m>=0).*(meshes.m<=par.ABS.*meshes.k).*(par.mortwedge/PI);
EVm = reshape(reshape(RBaux(:).*mutil_c,[mpar.nm*mpar.nk mpar.nh])*P',[mpar.nm,mpar.nk mpar.nh]);

[c_a_star,m_a_star,k_a_star,c_n_star,m_n_star] = EGM_policyupdate(EVm,EVk,Qminus,PIminus,RBminus,inc,meshes,grid,par,mpar);

%% Update Value Functions
meshaux=meshes;
meshaux.h(:,:,end)=1000;

EV    = reshape(V,[mpar.nm*mpar.nk, mpar.nh])*P';
Vnext = griddedInterpolant(meshaux.m,meshaux.k,meshaux.h,reshape(EV,[mpar.nm mpar.nk mpar.nh]));


VA    = util(c_a_star) + par.beta*Vnext(m_a_star,k_a_star,meshaux.h);
VN    = util(c_n_star) + par.beta*Vnext(m_n_star,meshaux.k,meshaux.h);

AProb = max(min(1./(1+exp(-((VA-VN)-par.mu_chi)./par.sigma_chi)),1-1e-6),1e-6);
AC =  par.sigma_chi.*((1-AProb(:)).*log(1-AProb(:)) + AProb(:).*log(AProb(:)))...
    + par.mu_chi.*AProb(:) -  (par.mu_chi-par.sigma_chi*log(1+exp(par.mu_chi/par.sigma_chi)));
AProb = AProb.*par.nu;
AC    = AC.*par.nu;

mutil = @(c)(1./(c.^par.xi));

MU = AProb(:).*c_a_star(:)+(1-AProb(:)).*c_n_star(:);
MU = mutil(MU);

%% Differences for distributions

% Transitions: find next smallest on-grid value for money and capital choices
[Dist_m_a,idm_a] = genweight(m_a_star,grid.m);
[Dist_m_n,idm_n] = genweight(m_n_star,grid.m);
[Dist_k,idk_a]   = genweight(k_a_star,grid.k);

% Iterate Distribution function forward
JD_new = JDIteration_mex(reshape(JDminus,[mpar.nm, mpar.nk, mpar.nh]),...
    int32(idm_n),int32(idm_a),int32(idk_a),Dist_m_n,Dist_m_a,Dist_k,...
    reshape(AProb,[mpar.nm, mpar.nk, mpar.nh]),P);

JD = reshape(JD_new(:),[mpar.nm,mpar.nk,mpar.nh]);

%% Third Set: Government Budget constraint
B = (grid.B+par.ABS*grid.K)*...
    exp(par.rho_B * log((Bminus)/(grid.B)) ...
    + par.rho_B * log(RBminus/par.RB)...
    - (par.gamma_pi+par.rho_B) * log(PIminus/par.PI) ...
    - par.gamma_T * log(taxrev/targets.T));

marginal_b=squeeze(sum(sum(JD,2),3));
marginal_k=squeeze(sum(sum(JD,1),3));
BNEW  = sum(grid.m.*marginal_b');
KNEW  = sum(grid.k.*marginal_k);

QuantDiff=[ KNEW-K,BNEW-B];

end
function [ weight,index ] = genweight( x,xgrid )
% function: GENWEIGHT generates weights and indexes used for linear interpolation
%
% X: Points at which function is to be interpolated.
% xgrid: grid points at which function is measured
% no extrapolation allowed
[~,index] = histc(x,xgrid);
index(x<=xgrid(1))=1;
index(x>=xgrid(end))=length(xgrid)-1;

weight = (x-xgrid(index))./(xgrid(index+1)-xgrid(index)); % weight xm of higher gridpoint
weight(weight<=0) = 1.e-16; % no extrapolation
weight(weight>=1) = 1-1.e-16; % no extrapolation

end  % function
