%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================

%% Initialize workspace and load directories
clear
clc
close all
Computername='NKIM'
starttime=clock;
set(0,'defaulttextinterpreter','latex')
addpath(genpath('functions'))
addpath(genpath('latex'))

%% 1) Load baseline steady state (run metascript_maintext.m first)
casename            = 'SS_BASELINE';

load(casename)

%% 2) Produce baseline results for uncertainty shocks
aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Simulation
sim_reitermodel

% Approximation Check
[denHaanTable]=denHaanSimulation(par, mpar, targets, grid, aggrshock);

filename=['latex/DenHaanAvgAbs_' casename '_' aggrshock '.tex'];
FID = fopen(filename, 'w');
fprintf(FID, '   %8.4f & %8.4f & %8.4f & %8.4f  \n', denHaanTable(1,:));
fclose(FID);
filename=['latex/DenHaanMaxAbs_' casename '_' aggrshock '.tex'];
FID = fopen(filename, 'w');
fprintf(FID, '   %8.4f & %8.4f & %8.4f & %8.4f  \n', denHaanTable(2,:));
fclose(FID);

close all
%% 2) Produce results for uncertainty shocks with flexible prices
casename='SS_BASELINE_FLEXPRICE';
par.prob_priceadj = 1/100; % average price duration of 4 quarters = 1/(1-par.prob_priceadj)
par.kappa         = (1-par.prob_priceadj)*(1-par.prob_priceadj*par.beta)/par.prob_priceadj;% Phillips-curve parameter (from Calvo prob.)

% 2.a) Baseline monetary and fiscal policy

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

% 2.b) No monetary and fiscal policy
par.gamma_pi = 0;
par.gamma_T  = 0;
par.rho_B    = 1;
par.theta_pi = 0;
par.rho_R    = 1;

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

% 2.c) Only monetary policy
par.gamma_pi = 0;
par.gamma_T  = 0;
par.rho_B    = 0.86;
par.theta_pi = 1.25;
par.rho_R    = 0.8;

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

close all
%% 3) Produce results for uncertainty shocks without stabilization policies
casename='SS_BASELINE';
% Phillips Curve
par.prob_priceadj = 3/4; % average price duration of 4 quarters = 1/(1-par.prob_priceadj)
par.kappa         = (1-par.prob_priceadj)*(1-par.prob_priceadj*par.beta)/par.prob_priceadj;% Phillips-curve parameter (from Calvo prob.)

% No monetary and fiscal policy
par.gamma_pi = 0;
par.gamma_T  = 0;
par.rho_B    = 1;
par.theta_pi = 0;
par.rho_R    = 1;

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

close all
%% 4) Produce results for TFP shocks
aggrshock     = 'TFP';
par.sigmaS=0.00969;
par.rhoS=0.95;

% Central Bank Policy
par.theta_pi    = 1.25; % Reaction to inflation
par.rho_R       = 0.8;  % Inertia
% Debt rule
par.gamma_pi    = 1.5;   % Reaction to inflation
par.gamma_T     = 0.5075; % Reaction to tax revenue
par.rho_B       = 0.86;  % Autocorrelation

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

% Simulation
sim_reitermodel

close all
%% 5) Produce results for uncertainty shocks with asset backed liquidity

% Initialize workspace and load directories
clear
clc
close all
Computername='NKIM'
starttime=clock;
set(0,'defaulttextinterpreter','latex')
addpath(genpath('functions'))
addpath(genpath('latex'))

% 5.a) Find new steady state
casename            = 'SS_BASELINE_ABS10';
% Set parameters
defineSS_pars
par.ABS = 0.10;     % Loan to value ratio max.
par.Q  = 1*(1-par.ABS);

mainskript_steadystate

% 5.b) Produce results for uncertainty shocks
aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

plot_IRFs

close all
%% 6.a) Robustness A
clear
casename            = 'SS_BASELINE';
load(casename)

% 6.a) Discount factor in Phillips Curve
par.beta_PC=par.RB^-1;
name_1  =['DISCOLIQUID'];
legend_1=['Disc. liquid (\beta^{FIRM}=' num2str(par.beta_PC) ')'];

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

% 6.a) Discount factor in Phillips Curve
par.beta_PC=0.7726;
name_2  =['DISCOENTR'];
legend_2=['Disc. entrep. (\beta^{FIRM}=' num2str(par.beta_PC) ')'];

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_2],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='disco';
legend_baseline=['Baseline (\beta^{FIRM}=' num2str(par.beta) ')'];

plot_IRFs_robustness_figure23

%% 6.b) Robustness A
clear

% 6.b) Discount factor low
% Find new steady state
casename            = 'SS_BASELINE_XI3';
% Set parameters
defineSS_pars
par.xi=3;
par.beta=0.9820;
par.sigma_chi= 225;
par.probitratio= 3.2000;
par.mu_chi      = par.probitratio*par.sigma_chi;
name_1  =['XI3'];
legend_1=['\xi = ' num2str(par.xi)];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

% 6.b) Discount factor high
% Find new steady state

casename            = 'SS_BASELINE_XI5';
% Set parameters
defineSS_pars
par.xi=5;
par.beta=0.976;
par.sigma_chi= 500000;
par.probitratio= 2.825;
par.mu_chi      = par.probitratio*par.sigma_chi;
name_2  =['XI5'];
legend_2=['\xi = ' num2str(par.xi) ];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_2],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='risk_aversion';
legend_baseline=['Baseline (\xi = 4 )'];

plot_IRFs_robustness_figure23

%% 6.c) Robustness A
clear

% 6.c) Frisch elasticity low
% Find new steady state
casename            = 'SS_BASELINE_GAMMA125';
% Set parameters
defineSS_pars
par.gamma = 1.25;
name_1  =['GAMMA125'];
legend_1=['Frisch elasticity = ' num2str(1/par.gamma) ];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

% 6.c) Frisch elasticity high
% Find new steady state

casename            = 'SS_BASELINE_GAMMA075';
% Set parameters
defineSS_pars
par.gamma = 0.75;
name_2  =['GAMMA075'];
legend_2=['Frisch elasticity = ' num2str(1/par.gamma) ];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_2],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='frisch_elasticity';
legend_baseline=['Baseline (Frisch elast.=1)'];

plot_IRFs_robustness_figure23

%% 7) Robustness B
clear

% 7.a) Low variance of portfolio adj costs
% Find new steady state
casename            = 'SS_BASELINE_LOWVAR';
% Set parameters
defineSS_pars
par.sigma_chi   = 2250;      % Variance of logit distribution of adj costs
par.logitratio  = 2.825;    
par.mu_chi      = par.logitratio*par.sigma_chi; % Mean of logit distribution of adj costs
name_1  =['LOWVAR'];
legend_1=['Low variance (0.1* \sigma_\chi)'];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='lowvar';
legend_baseline=['Baseline'];

plot_IRFs_robustness_figure24

% 7.b) High variance of portfolio adj costs
% Find new steady state
clear
casename            = 'SS_BASELINE_HIGHVAR';
% Set parameters
defineSS_pars
par.sigma_chi   = 2250000;      % Variance of logit distribution of adj costs
par.logitratio  = 2.825;    
par.mu_chi      = par.logitratio*par.sigma_chi; % Mean of logit distribution of adj costs
name_1  =['HIGHVAR'];
legend_1=['High variance (100* \sigma_\chi)'];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='highvar';
legend_baseline=['Baseline'];

plot_IRFs_robustness_figure24

% 7.c) High liquidity
% Find new steady state
clear
casename            = 'SS_BASELINE_HIGHLIQUIDITY';
% Set parameters
defineSS_pars
par.sigma_chi   = 22500;      % Variance of logit distribution of adj costs
par.logitratio  = 1.4125;    
par.mu_chi      = par.logitratio*par.sigma_chi; % Mean of logit distribution of adj costs
name_1  =['HIGHLIQUIDITY'];
legend_1=['20% adj. probability'];

mainskript_steadystate

aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Produce IRFs
mpar.maxlag=40; % Quarters

x0=zeros(numstates,1);
x0(end)=par.sigmaS;

MX=[eye(length(x0));gx];
IRF_state_sparse=[];
x=x0;

for t=1:mpar.maxlag
    IRF_state_sparse(:,t)=(MX*x)';
    x=hx*x;
end

plot_IRFs_prep

save(['IRF_data_' name_1],'IRF_Y','IRF_C','IRF_I','IRF_G','Deficit','IRF_K','IRF_M','IRF_N','IRF_S','IRF_PI','IRF_RB','IRF_RBREAL','IRF_Q','IRF_D','IRF_LP')

robustness='highliquidity';
legend_baseline=['Baseline (5.6% adj. prob.)'];

plot_IRFs_robustness_figure24

%% Find smaller steady state for Krusell-Smith and produce LoM
% Find new steady state
casename            = 'SS_BASELINE_KS';
% Set parameters
defineSS_pars
% Idiosyncratic States
mpar.nm         = 50;
mpar.nk         = 50;
mpar.nh         = 16;

mainskript_steadystate


aggrshock           = 'Uncertainty';
par.rhoS            = 0.839;    % Persistence of variance
par.sigmaS          = 0.539;    % STD of variance shocks

% Produce matrices to reduce state-space
mpar.maxdim=12;

mainskript_statereduc

% Initialize System
disp('Computing system for SS.');

F = @(a,b,c,d)Fsys(a,b,c,d,Xss,Yss,Gamma_state,Gamma_control,InvGamma,par,mpar,grid,targets,Copula,P_H,aggrshock,oc);
tic
[Fss,LHS,RHS,Distr] = F(State,State_m,Contr,Contr_m);
toc

% Solve RE via Schmitt-Grohe-Uribe Form
[hx,gx,F1,F2,F3,F4,par] = SGU_solver(F, numstates,numcontrols,Gamma_state,oc,mpar,par,grid);

% Simulation
sim_reitermodel
