function [ c_star,Resource,cons_list, res_list,mon_list,cap_list ] = EGM_Step3(EMU,grid,inc,m_a_star,c_n_aux,mpar,par)
%EGM_Step3 returns the resources (res_list), consumption (cons_list)
% and money policy (mon_list) for given capital choice (cap_list).
% For k'=0, there doesn't need to be a unique corresponding m*. We get a
% list of consumption choices and resources for money choices m'<m* (mon_list) and cap
% choices k'=0 (cap_list) and merge them with consumption choices and
% resources that we obtain if capital constraint doesn't bind next period.
%
% c_star: optimal consumption policy as function of k',s*h,M,K (both
% constraints do not bind)
%
% Resource: required resource for c_star
%
% cons_list: optimal consumption policy if a) only k>=0 binds and b) both
% constraints do not bind
%
% res_list: Required resorces for cons_list
% c_n_aux: consumption in t as function of t+1 grid (constrained version)

% Copyright (c) 2018-05-27
% Christian Bayer, Ralph Luetticke, Lien Pham-Dao, and Volker Tjaden

%% Constraints for money and capital are not binding
EMU=reshape(EMU,[mpar.nm, mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK]);

% Interpolation of psi-function at m*_n(m,k)
[~,idx]=histc(m_a_star,grid.m); % find indexes on grid next smallest to optimal policy
idx(m_a_star<=grid.m(1))=1; %if below minimum
idx(m_a_star>=grid.m(end))=mpar.nm-1; %if above maximum
step=diff(grid.m); %Stepsize on grid
s = (m_a_star - grid.m(idx))./step(idx);%Distance of optimal policy to next grid point

aux_index=(0:(mpar.nk*mpar.ns*mpar.nh*mpar.nM*mpar.nK)-1)*mpar.nm; %aux for linear indexes
aux3=EMU(idx(:)+aux_index(:)); %calculate linear indexes

% Interpolate EMU(m',k',s'*h',M',K') over m*_n(k'), m-dim is dropped
EMU_star = aux3 + s(:).*(EMU(idx(:)+aux_index(:)+1)-aux3); %linear interpolation

c_star = 1./(EMU_star.^(1/par.xi));
cap_expenditure = shiftdim(inc.capital(1,:,:,:,:));
auxL=shiftdim(inc.labor(1,:,:,:,:));
auxP=shiftdim(inc.profits(1,:,:,:,:));
% Resources that lead to capital choice k'
% = c + m*(k') + k' - w*h*N = value of todays cap and money holdings
Resource = c_star + m_a_star(:) + cap_expenditure(:) - auxL(:) - auxP(:);

c_star = reshape(c_star, [mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);
Resource = reshape(Resource, [mpar.nk mpar.ns*mpar.nh mpar.nM mpar.nK]);

%% Money constraint is not binding, but capital constraint is binding
m_star_zero=shiftdim(m_a_star(1,:,:,:)); % Money holdings that correspond to k'=0:  m*(k=0)

% Use consumption at k'=0 from constrained problem, when m' is on grid
aux_c=reshape(c_n_aux(:,1,:,:,:),[mpar.nm, mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
aux_inc=reshape(inc.labor(1,1,:,:,:)+inc.profits(1,1,:,:,:),[1 mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
cons_list=cell(mpar.nM*mpar.nK*mpar.ns*mpar.nh,1);
res_list=cell(mpar.nM*mpar.nK*mpar.ns*mpar.nh,1);
mon_list=cell(mpar.nM*mpar.nK*mpar.ns*mpar.nh,1);
cap_list=cell(mpar.nM*mpar.nK*mpar.ns*mpar.nh,1);


for j=1:mpar.ns*mpar.nh*mpar.nM*mpar.nK
    % When choosing zero capital holdings, HHs might still want to choose money
    % holdings smaller than m*(k'=0)
    if m_star_zero(j)>grid.m(1) 
        % Calculate consumption policies, when HHs chooses money holdings
        % lower than m*(k'=0) and capital holdings k'=0 and save them in cons_list
        log_index=grid.m<m_star_zero(j);
        % aux_c is the consumption policy under no cap. adj. (fix k�=0), for m�<m_a*(k'=0) 
        c_k_cons=aux_c(log_index,j); 
        cons_list{j}=c_k_cons; %Consumption at k'=0, m'<m_a*(0)  
        % Required Resources: Money choice + Consumption - labor income
        % Resources that lead to k'=0 and m'<m*(k'=0)
        res_list{j}=grid.m(log_index)' + c_k_cons - aux_inc(j);
        mon_list{j}=grid.m(log_index)';
        cap_list{j}=zeros(sum(log_index),1);
    end
end

%% Merge lists
c_star=reshape(c_star,[mpar.nk mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
m_a_star=reshape(m_a_star,[mpar.nk mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
Resource=reshape(Resource,[mpar.nk mpar.ns*mpar.nh*mpar.nM*mpar.nK]);
for j=1:mpar.nM*mpar.nK*mpar.ns*mpar.nh
    cons_list{j}=[cons_list{j} ;c_star(:,j)];
    res_list{j}=[ res_list{j};Resource(:,j)];
    mon_list{j}=[ mon_list{j};m_a_star(:,j)];
    cap_list{j}=[cap_list{j}; grid.k'];
end


end

