function [c_new,m_star,distPOL] = policies_SS(c_guess, grid, inc,RBRB,P,mpar,par)
% POLICIES solves for the household policies for consumption, money and
% capital holdings in the adjustment and non-adjustment case. Given returns
% {q,R_fc,PI} and interpolation matrices {P,P_K,P_M,TT} a variant of the
% endogeneous gridpoint method is used to iterate on the first order
% conditions until convergence.
% Policies in adjustment case:
% {c_a_new,m_a_star,cap_a_star}
% Policies in non-adjustment case:
% {c_n_new,m_n_star}

% Copyright (c) 2014-02-28
% Christian Bayer, Ralph L�tticke, Lien Pham-Dao, and Volker Tjaden
%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================

%% Apply EGM to solve for optimal policies and marginal utilities
money_expense  = repmat(grid.m',[1 mpar.nh*mpar.ns]);
distC = 99999;

count=0;
while max([distC])>mpar.crit
    count=count+1;

    %% Step 1: Update policies for only money adjustment
    mutil_c = 1./(c_guess.^par.xi); % marginal utility at consumption policy no adjustment

    mutil_c=RBRB.*mutil_c; %take return on money into account
    aux=reshape(permute(mutil_c,[2 1]),[mpar.ns*mpar.nh mpar.nm]);
    % form expectations
    EMU_aux = par.beta*permute(reshape(P*aux,[mpar.ns*mpar.nh mpar.nm]),[2 1]);

    c_aux = 1./(EMU_aux.^(1/par.xi));

    % Take borrowing constraint into account
    [c_new,m_star]=EGM_Step1_b(grid,inc,money_expense,c_aux,mpar,par);

    m_star(m_star>grid.m(end)) = grid.m(end);
   
    %% Step 6: Check convergence of policies
    distC = max((abs(c_guess(:)-c_new(:))));
    
    % Update c policy guesses
    c_guess=c_new;

end
distPOL=[distC];
end

%% SUBFUNCTIONS

function [c_update,m_update]=EGM_Step1_b(grid,inc,money_expense,c_aux,mpar,par)
%%EGM_Step1_b computes the optimal consumption and corresponding optimal money
% holdings in case the capital stock cannot be adjusted by taking the budget
% constraint into account.
% c_update(m,k,s*h,M,K):    Update for consumption policy under no-adj.
% m_update(m,k,s*h,M,K):    Update for money policy under no-adj.

%% EGM: Calculate assets consistent with choices being (m')
% Calculate initial money position from the budget constraint,
% that leads to the optimal consumption choice
m_star = (c_aux + money_expense - inc.labor - inc.profits);
m_star = m_star./par.RB;

% Identify binding constraints
binding_constraints = money_expense < repmat(m_star(1,:),[mpar.nm 1 ]);

% Consumption when drawing assets m' to zero: Eat all Resources
Resource = inc.labor  + inc.money + inc.profits;

%% Next step: Interpolate w_guess and c_guess from new k-grids
% using c(s,h,k',K), k(s,h,k'K

m_star = reshape(m_star,[mpar.nm mpar.ns*mpar.nh]);
c_aux= reshape(c_aux,[mpar.nm mpar.ns*mpar.nh]);

%Interpolate grid.m and c_n_aux defined on m_star_n over grid.m
[c_update, m_update]=egm1b_aux_mex(grid.m,m_star,c_aux);

c_update = reshape(c_update,[mpar.nm, mpar.ns*mpar.nh]);
m_update = reshape(m_update,[mpar.nm, mpar.ns*mpar.nh]);

c_update(binding_constraints) = Resource(binding_constraints)-grid.m(1);
m_update(binding_constraints) = min(grid.m);

end
