function [Difference,LHS,RHS,JD_new,c_star,m_star,P]  = Fsys(State,Stateminus,...
    Control_sparse,Controlminus_sparse,StateSS,...
    ControlSS,Gamma_state,Gamma_control,InvGamma,Copula,...
    par,mpar,grid,targets,P,aggrshock,oc,os)
% System of equations written in Schmitt-Grohé-Uribe generic form with states and controls
% STATE: Vector of state variables t+1 (only marginal distributions for histogram)
% STATEMINUS: Vector of state variables t (only marginal distributions for histogram)
% CONTROL: Vector of state variables t+1 (only coefficients of sparse polynomial)
% CONTROLMINUS: Vector of state variables t (only coefficients of sparse polynomial)
% STATESS and CONTROLSS: Value of the state and control variables in steady
% state. For the Value functions these are at full grids.
% GAMMA_STATE: Mapping such that perturbationof marginals are still
% distributions (sum to 1).
% GAMMA_CONTROL: Values of the polynomial base at all nodes to map
% sparse coefficient changes to full grid
% INVGAMMA: Projection of Value functions etc. to Coeffeicent space for
% sparse polynomials.
% PAR, MPAR: Model and numerical parameters (structure)
% GRID: Liquid, illiquid and productivity grid
% TARGETS: Stores targets for government policy
% COPULA: Interpolant that allows to map marginals back to full-grid
% distribuitions
% P: steady state transition matrix
% aggrshock: sets wether the Aggregate shock is TFP or uncertainty
%
% Authors: Christian Bayer and Ralph C. Luetticke, Bonn and London, March
% 2017%
% =========================================================================
% Part of the Matlab code to accompany the paper
% 'Precautionary Savings, Illiquid Assets, and the Aggregate Consequences of
%  Shocks to Household Income Risk', Bonn mimeo
% http://wiwi.uni-bonn.de/hump/wp.html
% =========================================================================
%
% Permission is hereby granted, free of charge, to any person obtaining a
% copy of this software and associated documentation files (the "Software"),
% to deal in the Software without restriction, including without limitation
% the rights to use, copy, modify, merge, publish, distribute, sublicense,
% and/or sell copies of the Software, and to permit persons to whom the
% Software is furnished to do so, subject to the following conditions:
%
% The most recent version or successor of the above-mentioned paper is
% properly cited in all work and publications that benefit from insights
% derived from the Software.
% The above copyright notice and this permission notice shall be included
% in all copies or substantial portions of the Software.
%
% THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
% OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
% FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
% THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
% LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
% DEALINGS IN THE SOFTWARE.
%
% =========================================================================


%% Initializations
mutil = @(c)(1./(c.^par.xi));
invmutil = @(mu)((1./mu).^(1/par.xi));
% Generate meshes for b,k,h
[meshes.m,meshes.h] = ndgrid(grid.m,grid.h);

% Number of states, controls
nx   = mpar.numstates; % Number of states
ny   = mpar.numcontrols; % number of Controls
NxNx = nx-os; % Number of states without aggregate
NN   = mpar.nm*mpar.nh; % Number of points in the full grid

% Initialize LHS and RHS
LHS  = zeros(nx+ny,1);
RHS  = zeros(nx+ny,1);

%% Indexes for LHS/RHS
% Indexes for Controls
mutil_cind = 1:NN;
Qind  = 1*NN+1;
PIind = 1*NN+2;
Yind  = 1*NN+3;
Gind  = 1*NN+4;
Wind  = 1*NN+5;
Rind  = 1*NN+6;
Profitind  = 1*NN+7;
Nind  = 1*NN+8;
Tind  = 1*NN+9;
Kind  = 1*NN+10;
Bind  = 1*NN+11;

% Indexes for States
marginal_mind = (1:mpar.nm-1);
marginal_hind = (mpar.nm-1 + (1:(mpar.nh-2)));

RBind = NxNx+1;
BtoKind  = NxNx+2;
Sind  = NxNx+3;

%% Control Variables (Change Value functions according to sparse polynomial)
Control      = ControlSS .* (1+Gamma_control*(Control_sparse));
Controlminus = ControlSS .* (1+Gamma_control*(Controlminus_sparse));

Control(end-oc+1:end)       = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Control_sparse);
Controlminus(end-oc+1:end)  = ControlSS(end-oc+1:end) + Gamma_control(end-oc+1:end,:)*(Controlminus_sparse);


%% State Variables
% read out marginal histogramm in t+1, t
Distribution      = StateSS(1:end-os) + Gamma_state * State(1:NxNx);
Distributionminus = StateSS(1:end-os) + Gamma_state * Stateminus(1:NxNx);

% Aggregate Endogenous States
RB      = StateSS(end-2) + (State(end-2));
RBminus = StateSS(end-2) + (Stateminus(end-2));

% Aggregate Exogenous States
BtoK       = StateSS(end-1) + (State(end-1));
BtoKminus  = StateSS(end-1) + (Stateminus(end-1));

% Aggregate Exogenous States
S       = StateSS(end) + (State(end));
Sminus  = StateSS(end) + (Stateminus(end));

%% Split the Control vector into items with names
% Controls
mutil_c       = mutil(Control(mutil_cind));
mutil_cminus  = mutil(Controlminus(mutil_cind));

% Aggregate Controls (t+1)
PI = exp(Control(PIind));
Y  = exp(Control(Yind ));
K  = exp(Control(Kind ));
Q  = exp(Control(Qind ));
R  = exp(Control(Rind ));
B = exp(Control(Bind ));

% Aggregate Controls (t)
PIminus = exp(Controlminus(PIind));
Qminus  = exp(Controlminus(Qind ));
Yminus  = exp(Controlminus(Yind ));
Gminus  = exp(Controlminus(Gind ));
Wminus  = exp(Controlminus(Wind ));
Rminus  = exp(Controlminus(Rind ));
Profitminus  = exp(Controlminus(Profitind ));
Nminus  = exp(Controlminus(Nind ));
Tminus  = exp(Controlminus(Tind ));
Kminus  = exp(Controlminus(Kind ));
Bminus  = exp(Controlminus(Bind ));

%% Write LHS values
% Controls
LHS(nx+mutil_cind) = invmutil(mutil_cminus);
LHS(nx+Qind)       = (Qminus);
LHS(nx+Yind)       = (Yminus);

LHS(nx+Gind)       = (Gminus);
LHS(nx+Wind)       = (Wminus);
LHS(nx+Rind)       = (Rminus);
LHS(nx+Profitind)  = (Profitminus);
LHS(nx+Nind)       = (Nminus);
LHS(nx+Tind)       = (Tminus);
LHS(nx+Kind)       = (Kminus);
LHS(nx+Bind)       = (Bminus);

% States
% Marginal Distributions (Marginal histograms)
LHS(marginal_mind) = Distribution(1:mpar.nm-1);
LHS(marginal_hind) = Distribution(mpar.nm+(1:mpar.nh-2));

LHS(RBind)         = (RB);
LHS(Sind)          = (S);
LHS(BtoKind)       = (BtoK);

% take into account that RB and B are in logs
RB=exp(RB); RBminus=exp(RBminus);
BtoK=exp(BtoK); BtoKminus=exp(BtoKminus);

%% Set of Differences for exogenous process
RHS(Sind) = (par.rhoS * (Sminus));

switch(aggrshock)
    case('TFP')
        TFP=exp(Sminus);
    case('Uncertainty')
        TFP=1;
        % Tauchen style for Probability distribution next period
        [P,~,~] = ExTransitions(exp(Sminus),grid,mpar,par);
    
end

marginal_mminus = Distributionminus(1:mpar.nm)';
marginal_hminus = Distributionminus(mpar.nm+(1:mpar.nh))';

Hminus  = grid.h(1:end-1)*marginal_hminus(1:end-1)'; %Last column is entrepreneurs.

RHS(nx+Kind)= grid.m*marginal_mminus';

% Calculate joint distributions
cumdist = zeros(mpar.nm+1,mpar.nh+1);
cumdist(2:end,2:end) = Copula({cumsum(marginal_mminus),cumsum(marginal_hminus)});
JDminus = diff(diff(cumdist,1,1),1,2);

% Generate meshes for b,k,h
[meshes.m,meshes.h] = ndgrid(grid.m,grid.h);

%%
mc             =  par.mu- (par.beta * log(PI)*Y/Yminus - log(PIminus))/par.kappa;

RHS(nx+Nind)    =  (par.tau*TFP*par.alpha*Kminus.^(1-par.alpha).*mc).^(1/(1-par.alpha+par.gamma));

RHS(nx+Yind)    = (TFP*(Nminus).^(par.alpha).*Kminus.^(1-par.alpha));

% Wage Rate
RHS(nx+Wind) = TFP *par.alpha       *mc.* (Kminus./(Nminus)).^(1-par.alpha);
% Return on Capital
RHS(nx+Rind) = TFP *(1-par.alpha)   *mc.* ((Nminus)./Kminus).^(par.alpha)  - par.delta;

% Profits for Entrepreneurs
RHS(nx+Profitind) = (1-mc)*Yminus - Yminus.*(1/(1-par.mu))./par.kappa./2 .*log(PIminus).^2 + 1/2*par.phi*((K-Kminus).^2)./Kminus;

%% Wages net of leisure services
WW=par.gamma/(1+par.gamma)*(Nminus./Hminus)*Wminus*ones(mpar.nm,mpar.nh);
WW(:,end)=Profitminus*par.profitshare;

%% Incomes (grids)
inc.labor   = par.tau*WW.*(meshes.h);
inc.money   = (Rminus+Qminus+BtoKminus*RBminus/PIminus)*meshes.m;

%% First Set: Value Functions, Marginal Values
%% Update policies

Raux = (R+Q+BtoK*RB/PI)/(Qminus+BtoK); % Expected marginal utility at consumption policy (w &w/o adjustment)
EVm = reshape(reshape(Raux(:).*mutil_c,[mpar.nm mpar.nh])*P',[mpar.nm, mpar.nh]);

[c_star,m_star] = EGM_policyupdate(EVm,Rminus,Qminus,BtoK,BtoKminus,RBminus,PIminus,inc,meshes,grid,par,mpar);


%% Update Marginal Value Bonds
mutil_c_aux = mutil(c_star); % marginal utility at consumption policy no adjustment
RHS(nx+mutil_cind) = invmutil(mutil_c_aux(:)); % Write Marginal Utility to RHS of F

%% Differences for distributions
% find next smallest on-grid value for money choices
weight11  = zeros(mpar.nm, mpar.nh,mpar.nh);
weight12  = zeros(mpar.nm, mpar.nh,mpar.nh);

% Adjustment case
[Dist_m,idm] = genweight(m_star,grid.m);

idm=repmat(idm(:),[1 mpar.nh]);
idh=kron(1:mpar.nh,ones(1,mpar.nm*mpar.nh));

index11 = sub2ind([mpar.nm mpar.nh],idm(:),idh(:));
index12 = sub2ind([mpar.nm mpar.nh],idm(:)+1,idh(:));

for hh=1:mpar.nh
    
    %Corresponding weights
    weight11_aux = (1-Dist_m(:,hh));
    weight12_aux =  (Dist_m(:,hh));
    
    % Dimensions (mxk,h',h)
    weight11(:,:,hh)=weight11_aux(:)*P(hh,:);
    weight12(:,:,hh)=weight12_aux(:)*P(hh,:);
end

weight11=permute(weight11,[1 3 2]);
weight12=permute(weight12,[1 3 2]);

rowindex=repmat(1:mpar.nm*mpar.nh,[1 2*mpar.nh]);

H=sparse(rowindex,[index11(:); index12(:)],...
    [weight11(:); weight12(:)],mpar.nm*mpar.nh,mpar.nm*mpar.nh); % mu'(h',k'), a without interest


JD_new=JDminus(:)'*H;
JD_new = reshape(JD_new(:),[mpar.nm,mpar.nh]);
% Next period marginal histograms
% liquid assets
aux_m = squeeze(sum(JD_new,2));
RHS(marginal_mind) = aux_m(1:end-1); %Leave out last state
% human capital
aux_h = squeeze(sum(JD_new,1));
RHS(marginal_hind) = aux_h(1:end-2); %Leave out last state & entrepreneurs


%% Third Set: Government Budget constraint
% Return on bonds (Taylor Rule)
RHS(RBind) = log(par.RB) + par.rho_R* log(RBminus/par.RB)  + log(PIminus/par.PI).*((1-par.rho_R)*par.theta_pi);

taxrevenue =(1-par.tau).*Wminus.*Nminus +(1-par.tau).*Profitminus;

% Inflation jumps to equilibrate real bond supply and demand
RHS(nx+PIind) = par.rho_B * log((Bminus)/(targets.B)) ...
    + par.rho_B * log(RBminus/par.RB)...
    - (par.rho_B+par.gamma_pi) * log(PIminus) ...
    - par.gamma_T * log((Tminus)/(targets.T));
LHS(nx+PIind) = log((B)/(targets.B));

RHS(nx+Bind) = BtoKminus*Kminus;

LHS(BtoKind) = RB/PI;
RHS(BtoKind) = R/Qminus+Q/Qminus;

% Government expenditures
RHS(nx+Gind) =  B - Bminus*RBminus/PIminus + Tminus;

RHS(nx+Tind) = taxrevenue;

RHS(nx+Qind)=(par.phi*(K./Kminus-1)+1);

%% Difference
Difference = InvGamma(:,:)*((LHS-RHS)./[ones(nx,1);(ControlSS(1:end-oc));ones(oc,1)]);


end
function [ weight,index ] = genweight( x,xgrid )
% function: GENWEIGHT generates weights and indexes used for linear interpolation
%
% X: Points at which function is to be interpolated.
% xgrid: grid points at which function is measured
% no extrapolation allowed
[~,index] = histc(x,xgrid);
index(x<=xgrid(1))=1;
index(x>=xgrid(end))=length(xgrid)-1;

weight = (x-xgrid(index))./(xgrid(index+1)-xgrid(index)); % weight xm of higher gridpoint
weight(weight<=0) = 1.e-16; % no extrapolation
weight(weight>=1) = 1-1.e-16; % no extrapolation

end  % function
